/*
 * 28.08.2004 by Andreas Grunewald
 */
package org.jd3lib;

import java.lang.reflect.Method;

/**
 * The Baseclass for all Id3FrameFields One major importance is the setValue
 * method it uses reflection to set the implementing Field Values this works
 * only if the Class who extends the ID3Field has it's set Methods Called
 * setField and nothing else.
 * 
 * @author Andreas Grunewald
 *  
 */
public abstract class Id3Field {

    /**
     * returns the byte[] representation of this field, it can be directly
     * written to the ID3Tag with this.
     */
    public abstract byte[] getBytes();

    /**
     * This is a generic access method. You can set the Value by putting the
     * right type of Object in. Check the FieldClass you are setting to see
     * which Types are accepted
     * 
     * @param input
     *            has to be an Object which fits the field.
     * @throws IllegalArgumentExeption
     *             if Object is rejected.
     */
    public void setValue(Object input) throws IllegalArgumentException {
        Class[] params = { input.getClass() };
        Object[] args = { input };
        Method setTypeMethod;
        try {
            setTypeMethod = this.getClass().getMethod("setField", params);
            setTypeMethod.invoke(this, args);
        } catch (NoSuchMethodException e) {
            // belongs to getMethod
            throw new IllegalArgumentException(
                    "The "
                            + this.getClass().getName()
                            + " does not support setting it's value with arguments of the type "
                            + input.getClass().getName());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * @deprecated will be removed for release
     */
    protected void setValueOld(Object input) throws IllegalArgumentException {
        input.getClass();
    }

    /**
     * Returns the Object representation of the Field, can be basic types,
     * strings or even complex objects like encoding or URLs check the Field
     * itself for access information
     */
    public abstract Object getValue();
}