/*
 * $Id: Id3Frame.java,v 1.10 2005/04/27 08:35:59 gruni Exp $
 * $Author: gruni $ 
 * 
 * $Log: Id3Frame.java,v $
 * Revision 1.10  2005/04/27 08:35:59  gruni
 * changes due to refactoring
 *
 */
package org.jd3lib;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;

/**
 * @author Andreas Grunewald
 * 
 * LATER 1.0 Write Documentation DONEXT comeup with a better size Calculation
 * sheme
 */
public abstract class Id3Frame {

  private Id3v2FrameHeader theHeader;

  public Id3Frame(ByteBuffer frameData, Id3v2FrameHeader header) {

    if (frameData == null) {
      //FIXME Is this really a good way to create a new frame ?
      initializeNew();
    } else {
      //FIXME Check that all Frames accept 1 byte long data;
      //FIXME Convert all frames to stream reading like APIC
      // (04.09.25.23:24)
      theHeader = header;
      initializeExisting(frameData, header);
    }
  }

  /*
   * public Id3Frame(InputStream tagData, Id3v2FrameHeader header) {
   * 
   * if (tagData == null) { //FIXME Is this really a good way to create a new
   * frame ? initializeNew(); } else { //FIXME Check that all Frames accept 1
   * byte long data; //FIXME Convert all frames to stream reading like APIC //
   * (04.09.25.23:24) theHeader = header; initializeExisting(tagData, header); } }
   */

  /**
   * Creates a new Frame by first creating the appropriate Header via reflection
   * and then initializing the Frame, it's also taken care of updateing the size
   * and the default flags.
   */
  private void initializeNew() {
    if (this.getClass().getName().startsWith(Id3Frame.class.getName())) {
      int end = this.getClass().getName().length();
      int start = end - 4;
      String out = this.getClass().getName().substring(start, end);
      // System.out.println("INITNEW:" + out);
      theHeader = new Id3v2FrameHeader(out);
      theHeader.setFlags(this.getDefaultFlags());
      this.initializeNew(theHeader);
      this.updateSize();
    } else {
      throw new RuntimeException(
          "The Implemented Class does not comply with the Naming conventions"
              + "the Class needs to end with the 4 letters of the FrameID");
    }

  }

  /**
   * Contains the default flags for a new frame, this method can be overridden
   * by an Extending class to declare the default Flags for a Implementing Frame
   */
  public byte[] getDefaultFlags() {
    return new byte[] { 0x00, 0x00 };
  }

  /**
   * Initialize a newly created Frame
   * 
   * @param header
   */
  protected abstract void initializeNew(Id3v2FrameHeader header);

  /**
   * Returns the FrameID
   */
  public String getFrameID() {
    return theHeader.getFrameID();
  }

  /**
   * @return Returns the size.
   */
  public int getSize() {
    updateSize();
    return theHeader.getSize();
  }

  /**
   * @param frameData
   * @param header
   */
  protected abstract void initializeExisting(ByteBuffer frameData,
      Id3v2FrameHeader header);

  /**
   * Initialize a Frame with the given Data
   * 
   * @param tagData
   *          the Inputstream where the Data is read from
   * @param header2
   *          The Header to this frame holding Type and size information
   */
  protected abstract void initializeExisting(InputStream tagData,
      Id3v2FrameHeader header2);

  /**
   * Returns a String Representation of the Frame with all it's Fields
   */
  protected abstract String getFrameString();

  public void updateSize() {
    theHeader.setSize(getFrameData().size());
  }

  public ByteArrayOutputStream getAllData() {
    ByteArrayOutputStream theFrameData = getFrameData();
    ByteArrayOutputStream out = new ByteArrayOutputStream();
    theHeader.setSize(theFrameData.size());
    try {
      theHeader.getRawData().writeTo(out);
      theFrameData.writeTo(out);
    } catch (IOException e) {
      e.printStackTrace();
    }
    return out;
  }

  /**
   *  
   */
  protected abstract ByteArrayOutputStream getFrameData();

  /*
   * (non-Javadoc)
   * 
   * @see java.lang.Object#toString()
   */
  public String toString() {
    updateSize();
    return theHeader + getFrameString();
  }
}