/*
 * $Id: Id3v1Tag.java,v 1.4 2005/04/27 08:51:02 gruni Exp $
 * $Author: gruni $ 
 *
 * $Log: Id3v1Tag.java,v $
 * Revision 1.4  2005/04/27 08:51:02  gruni
 * minor corrections
 *
 * Revision 1.3  2005/01/27 17:09:43  gruni
 * uodated comments cleanup
 *
 * Revision 1.2  2004/12/16 16:59:52  gruni
 * a lot of updates switching to NIO and adding ID3v1
 *
 */
package org.jd3lib;

import java.io.ByteArrayOutputStream;
import java.nio.ByteBuffer;
import java.util.Arrays;

/*
 Copyright (C) 2001,2002 Jonathan Hilliker
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 Lesser General Public License for more details.
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
/**
 * This class reads and writes id3v1.1 tags from/to files. <br/>
 * 
 * <dl>
 * <dt><b>Version History: </b></dt>
 * <dt>2.0 - <small>2004.1214 by gruni </small></dt>
 * <dd>moved to next generation jd3lib and adapted interface</dd>
 * <dt>1.9.2 - <small>2002.1107 by gruni </small></dt>
 * <dd>-TrackNo. should now be read correctly if id3v1.1 Tag present. Should be
 * -1 if not.</dd>
 * <dt>1.9.1 - <small>2002.1023 by gruni </small></dt>
 * <dd>-Made sourcecode compliant to the Sun Coding Conventions</dd>
 * <dt>1.9 - <small>by helliker </small></dt>
 * </dl>
 * 
 * 
 * @author Jonathan Hilliker &&Andreas Grunewald
 * @version 2.0
 */

public class Id3v1Tag implements MetaData {

  /**
   * The size of an ID3v1 Tag
   */
  private static final int TAG_SIZE = 128;

  /**
   * The size of the Album
   */
  private static final int FIELD_SIZE = 30;

  /**
   * The size of the Comment in ID3v1.1
   */
  private static final int COMMENT_SIZE = 28;

  /**
   * The Track location
   */
  private static final int TRACK_LOCATION = 126;

  /**
   * The Genre location
   */
  private static final int GENRE_LOCATION = 127;

  /**
   * Maximum Number of genres ?
   */
  private static final int MAX_GENRE = 255;

  /**
   * Maximum Tracknumber ?
   */
  private static final int MAX_TRACK = 255;

  /**
   * Tag Start of ID3v1
   */
  private static final String TAG_ID = "TAG";

  private static final byte[] TAG = { 84, 65, 71 };

  /**
   * Gives Information if header Exists ?
   */
  private boolean headerExists = false;

  /**
   * The title
   */
  private String title;

  /**
   * The artist
   */
  private String artist;

  /**
   * The album
   */
  private String album;

  /**
   * The year
   */
  private String year;

  /**
   * The Comment
   */
  private String comment;

  /**
   * The Genre
   */
  private int genre;

  /**
   * The Track Number
   */
  private int track;

  /**
   * Create an id3v1tag from the file specified. If the file contains a tag, the
   * information is automatically extracted. Supports only id3v1.1 directly but
   * id3v1.0 tags will most likely work correctly too.
   * 
   * Song title 30 characters Artist 30 characters Album 30 characters Year 4
   * characters Comment 30 characters Genre 1 byte
   * 
   * @param mp3file
   *          the file to read/write the tag to
   * @exception FileNotFoundException
   *              if an error occurs
   * @exception IOException
   *              if an error occurs
   */
  public Id3v1Tag(ByteBuffer theBuffer) throws InstantiationException {

    byte[] header = new byte[3];
    theBuffer.get(header);

    // System.out.println(new String(header));
    // System.out.println("bow" + header[0] + " " + header[1] + " " +
    // header[2]);
    //System.out.println(Arrays.equals(header, TAG));
    if (theBuffer.limit() == 128 && Arrays.equals(header, TAG)) {
      byte[] tempArray = new byte[FIELD_SIZE];
      //System.out.println(theBuffer);
      theBuffer.get(tempArray);
      this.title = new String(tempArray);
      theBuffer.get(tempArray);
      this.artist = new String(tempArray);
      //System.out.println(theBuffer);
      theBuffer.get(tempArray);
      this.album = new String(tempArray);
      // System.out.println(theBuffer);
      tempArray = new byte[4];
      theBuffer.get(tempArray);
      this.year = new String(tempArray);
      // System.out.println(theBuffer);
      tempArray = new byte[FIELD_SIZE - 2];
      theBuffer.get(tempArray);
      this.comment = new String(tempArray);
      theBuffer.get(); //onebyte Zero irgnored
      this.track = theBuffer.get();
      this.genre = theBuffer.get();
      // System.out.println(theBuffer);

    } else {
      throw new InstantiationException(
          "Cant creat new tag because Tag is invalid");
    }

    //    RandomAccessFile in = null;
    //
    //    try {
    //      in = new RandomAccessFile(this.mp3, "r");
    //      this.headerExists = checkHeader(in);
    //
    //      if (this.headerExists) {
    //        readTag(in);
    //      }
    //    } finally {
    //      if (in != null) {
    //        in.close();
    //      }
    //    }
  }

  /**
   *  
   */
  public Id3v1Tag() {

    // DONEXT Create this constructor for new Tag creation
  }

  /**
   * @return
   */
  public ByteArrayOutputStream getTagData() {
    // LATER 0.1 Auto generated method stub
    return null;
  }

  /**
   * Writes the information in this tag to the file specified in the
   * constructor. If a tag does not exist, one will be created. If a tag already
   * exists, it will be overwritten.
   * 
   * @exception FileNotFoundException
   *              if an error occurs
   * @exception IOException
   *              if an error occurs
   */
  /*
   * public void writeTag() throws FileNotFoundException, IOException {
   * 
   * RandomAccessFile raf = null;
   * 
   * try { raf = new RandomAccessFile(mp3, "rw");
   * 
   * if (headerExists) { raf.seek(raf.length() - TAG_SIZE); } else {
   * raf.seek(raf.length()); }
   * 
   * raf.write(getBytes()); } finally { if (raf != null) { raf.close(); } }
   * 
   * headerExists = true; }
   */

  /**
   * Returns a binary representation of this id3v1 tag. It is in the correct
   * format according to the id3v1.1 specifications.
   * 
   * @return a binary representation of this id3v1 tag
   * @deprecated not used anymore
   */
  public byte[] getBytes() {
    byte[] tag = new byte[TAG_SIZE];
    int bytesCopied = 0;

    System.arraycopy(TAG_ID.getBytes(), 0, tag, bytesCopied, TAG_ID.length());
    bytesCopied += TAG_ID.length();
    System.arraycopy(title.getBytes(), 0, tag, bytesCopied, title.length());
    bytesCopied += FIELD_SIZE;
    System.arraycopy(artist.getBytes(), 0, tag, bytesCopied, artist.length());
    bytesCopied += FIELD_SIZE;
    System.arraycopy(album.getBytes(), 0, tag, bytesCopied, album.length());
    bytesCopied += FIELD_SIZE;
    //   System.arraycopy(year, 0, tag, bytesCopied, 4);
    bytesCopied += 4;
    System.arraycopy(comment.getBytes(), 0, tag, bytesCopied, comment.length());
    tag[TRACK_LOCATION] = (byte) track;
    tag[GENRE_LOCATION] = (byte) genre;

    return tag;
  }

  /**
   * Removes the id3v1 tag from the file specified in the constructor
   * 
   * @exception FileNotFoundException
   *              if an error occurs
   * @exception IOException
   *              if an error occurs
   */
  /*
   * public void removeTag() throws FileNotFoundException, IOException { if
   * (headerExists) { RandomAccessFile raf = null;
   * 
   * try { raf = new RandomAccessFile(mp3, "rw"); Long bufSize = new
   * Long(raf.length() - TAG_SIZE); // Possible loss of precision here, but I
   * doubt there will be // a 2 billion byte mp3 out there byte[] buf = new
   * byte[bufSize.intValue()];
   * 
   * if (raf.read(buf) != bufSize.intValue()) { throw new IOException("Error
   * encountered while removing tag"); }
   * 
   * raf.setLength(bufSize.longValue()); raf.seek(0); raf.write(buf); } finally {
   * if (raf != null) { raf.close(); } }
   * 
   * headerExists = false; } }
   */

  /**
   * Return the genre name based on the ID3/Nullsoft standards. If the genre
   * value is not valid, an empty String is returned.
   * 
   * @return return the genre name or null if the genre value is not valid
   */
  public String getGenre() {
    return NullsoftID3GenreTable.getGenre(genre);
  }

  /**
   * Attempt to set the genre value of this tag from the string specified. The
   * value returned is based on the ID3/Nullsoft standards. Returns true if a
   * match is found in the table and false otherwise.
   * 
   * @param str
   *          the string value of the genre to attempt to set
   * @return true if a match is found, false otherwise
   */
  public void setGenre(String str) {
    int result = NullsoftID3GenreTable.getGenre(str);

    if (result != -1) {
      genre = result;
    }
  }

  /**
   * Checks if a tag exists
   * 
   * @return true if a tag exists
   * @deprecated
   */
  public boolean tagExists() {
    return headerExists;
  }

  /**
   * Return the title field of the tag
   * 
   * @return the title field of the tag
   */
  public String getTitle() {
    return title.trim();
  }

  /**
   * Set the title field of the tag. The maximum size of the String is 30. If
   * the size exceeds the maximum size, the String will be truncated.
   * 
   * @param newTitle
   *          the title for the tag
   */
  public void setTitle(String newTitle) {
    if (newTitle.length() > FIELD_SIZE) {
      title = newTitle.substring(0, FIELD_SIZE);
    } else {
      title = newTitle;
    }
  }

  /**
   * Return the artist field of the tag
   * 
   * @return the artist field of the tag
   */
  public String getArtist() {
    return artist.trim();
  }

  /**
   * Set the artist field of the tag. The maximum size of the String is 30. If
   * the size exceeds the maximum size, the String will be truncated.
   * 
   * @param newArtist
   *          the artist for the tag
   */
  public void setArtist(String newArtist) {
    if (newArtist.length() > FIELD_SIZE) {
      artist = newArtist.substring(0, FIELD_SIZE);
    } else {
      artist = newArtist;
    }
  }

  /**
   * Return the album field of the tag
   * 
   * @return the album field of the tag
   */
  public String getAlbum() {
    return album.trim();
  }

  /**
   * Set the album field of the tag. The maximum size of the String is 30. If
   * the size exceeds the maximum size, the String will be truncated.
   * 
   * @param newAlbum
   *          the album for the tag
   */
  public void setAlbum(String newAlbum) {
    if (newAlbum.length() > FIELD_SIZE) {
      album = newAlbum.substring(0, FIELD_SIZE);
    } else {
      album = newAlbum;
    }
  }

  /**
   * Return the year field of the tag
   * 
   * @return the year field of the tag
   */
  public int getYear() {
    try {
      return Integer.parseInt(year);
    } catch (NumberFormatException e) {
      return -1;
    }
  }

  /**
   * Set the year field of the tag. The maximum size of the String is 4. If the
   * size exceeds the maximum size, the String will be truncated.
   * 
   * @param newYear
   *          the year for the tag
   */
  public void setYear(int newYear) {
    year = Integer.toString(newYear);
  }

  /**
   * Return the comment field of the tag
   * 
   * @return the comment field of the tag
   */
  public String getComment() {
    return comment.trim();
  }

  /**
   * Set the comment field of the tag. The maximum size of the String is 30. If
   * the size exceeds the maximum size, the String will be truncated.
   * 
   * @param newComment
   *          the comment of the tag
   */
  public void setComment(String newComment) {
    if (newComment.length() > COMMENT_SIZE) {
      comment = newComment.substring(0, COMMENT_SIZE);
    } else {
      comment = newComment;
    }
  }

  /**
   * Return the track field of the tag, or -1 if the tag does not exist.
   * 
   * @return the track field of the tag
   */
  public int getTrack() {
    return track;
  }

  /**
   * Set the track field of the tag. The track number has to be between 0 and
   * 255. If it is not, nothing will happen.
   * 
   * @param newTrack
   *          the track of the tag
   */
  public void setTrack(int newTrack) {
    if ((newTrack <= MAX_TRACK) && (newTrack >= 0)) {
      track = newTrack;
    }
  }

  /**
   * Return the genre field of the tag, -1 if tag doesn't exist
   * 
   * @return the genre field of the tag
   */
  public int getGenreNumber() {
    return genre;
  }

  /**
   * Set the genre field of the tag. This probably should not be greater than
   * 115, but supports values from 0-255.
   * 
   * @param newGenre
   *          the genre of the tag
   * @exception ID3FieldDataException
   *              if the value supplie is invalid
   */
  public void setGenre(short newGenre) {
    if ((newGenre <= MAX_GENRE) && (newGenre >= 0)) {
      genre = newGenre;
    } else {
      genre = -1;
    }
  }

  /**
   * Returns a String representation of this object. Contains all information
   * within the tag.
   * 
   * @return a String representation of this object
   */
  public String toString() {
    return "ID3v1.1\n " + "\nTitle:\t\t\t\t" + getTitle() + "\nArtist:\t\t\t\t"
        + getArtist() + "\nAlbum:\t\t\t\t" + getAlbum() + "\nYear:\t\t\t\t"
        + getYear() + "\nComment:\t\t\t" + getComment() + "\nTrack:\t\t\t\t"
        + getTrack() + "\nGenre:\t\t\t\t" + getGenreNumber() + "=" + getGenre();
  }

}// ID3v1Tag

