/*
 * 22.08.2004
 * by Andreas Grunewald
 */
package org.jd3lib;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;

import org.jd3lib.util.Coder;

/**
 * Generic ID3v2.x Header which is used for every ID3v2 Tag whic included
 * ID3v2.2, ID3v2.3 and ID3v2.4 as of 2005
 * 
 * @author Andreas Grunewald
 */
public class Id3v2TagHeader {

  protected boolean tagPresent;

  private byte versionMajor;

  private byte versionMinor;

  private int size;

  private byte flags;

  /**
   * @param headerData
   *          TODO deprecate this method when bytebuffer version is implemented
   * @deprecated this code is now moved into the Factory
   */
  public Id3v2TagHeader(byte[] headerData) {

    //ID3Header lesen
    //Detect if ID3v2 Header
    String Test = "" + (char) headerData[0] + (char) headerData[1]
        + (char) headerData[2];
    tagPresent = "ID3".equals(Test);
    //Detect Version Number
    versionMajor = headerData[3];
    versionMinor = headerData[4];
    flags = headerData[5];
    // Integer.toBinaryString());

    byte[] sizeArray = { headerData[6], headerData[7], headerData[8],
        headerData[9] };
    size = Coder.getSyncsafeIntegerValue(sizeArray);
  }

  /**
   * Creates emty header with following Setup tagPresent = valid; versionMajor =
   * 3; versionMinor = 0; flags = 0; size = 10;
   * 
   * @deprecated is not really needed, should be better via creating a new
   *             ID3vXTag
   *  
   */
  public Id3v2TagHeader(boolean valid) {
    this((byte) 3, (byte) 0, (byte) 0, 10);
    tagPresent = valid;
  }

  /**
   * @deprecated not needed after new Structure
   * @param headerData
   */
  public Id3v2TagHeader(ByteBuffer headerData) {

    // DONEXT Auto-generated constructor stub
  }

  /**
   * Creates a new Header with the given values
   * 
   * @param i
   * @param j
   * @param k
   * @param l
   */
  public Id3v2TagHeader(byte major, byte minor, byte inFlags, int inSize) {
    tagPresent = true;
    versionMajor = major;
    versionMinor = minor;
    flags = inFlags;
    size = inSize;
  }

  /**
   * @return
   */
  public byte[] writeBytes() {
    byte[] header = new byte[10];
    header[0] = (byte) 'I';
    header[1] = (byte) 'D';
    header[2] = (byte) '3';

    header[3] = versionMajor;
    header[4] = versionMinor;

    header[5] = flags;
    byte[] output = Coder.getSyncsafeBytes(getSize());
    header[6] = output[0];
    header[7] = output[1];
    header[8] = output[2];
    header[9] = output[3];

    /*
     * for (int i = 0; i < header.length; i++) System.out.print( header[i] + "
     * ");
     */
    return header;
  }

  /**
   * @return
   * @deprectated this does not work for Headers correctly
   */
  public boolean valid() {
    return tagPresent && versionMajor == 3;
  }

  /**
   * @return
   */
  public int getSize() {
    return size;
  }

  /*
   * (non-Javadoc)
   * 
   * @see java.lang.Object#toString()
   */
  public String toString() {
    return "Version: 2." + versionMajor + "." + versionMinor + "\n" + "Size: "
        + getSize() + "\n" + "Flags: " + Integer.toBinaryString(flags) + "\n";

  }

  /**
   * @param i
   */
  public void setSize(int i) {
    size = i;
  }

  /**
   * @return
   */
  public ByteArrayOutputStream getHeaderData() {
    ByteArrayOutputStream out = new ByteArrayOutputStream();
    try {
      out.write(this.writeBytes());
    } catch (IOException e) {
      e.printStackTrace();
    }
    return out;
  }
}