/*
 * $Id: Id3v2_3TagFactory.java,v 1.1 2005/04/27 08:56:48 gruni Exp $
 * $Author: gruni $ 
 * $Revision: 1.1 $
 * $Date: 2005/04/27 08:56:48 $
 * $Log: Id3v2_3TagFactory.java,v $
 * Revision 1.1  2005/04/27 08:56:48  gruni
 * Factory for Id3v2.3Tags
 *
 */
package org.jd3lib;

import java.nio.ByteBuffer;

import org.jd3lib.util.DebugOut;

/**
 * DONEXT Implement Basic Handling for most important frames. In ID3v2.3 Needed
 * Title, Artist, Album, Genra, Year, Track everything else Unknown binary.
 * 
 * @author Grunewald
 */
public class Id3v2_3TagFactory
    extends Id3v2TagFactory {

  public static final int headerSize = 10;

  public Id3v2_3TagFactory(byte versionMinor, byte flags, int size) {
    super((byte) 3, versionMinor, flags, size);
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.Id3v2_2TagFactory#Id3FramesList(java.nio.ByteBuffer)
   */
  protected Id3FrameList process(ByteBuffer tag) {
    Id3FrameList frames = new Id3FrameList();
    byte[] headbuf = new byte[headerSize];
    byte[] frame;
    while (tag.remaining() >= headerSize) {
      //System.out.println("Calling while");
      tag.get(headbuf);
      Id3v2FrameHeader head = new Id3v2_3FrameHeader(headbuf);
      DebugOut.println(head);
      //FIXME v1.0 Because of Problems with MP3s tagged with
      // MP3InfoExtension the head gets checked against MP3e which is the
      // padding when Tagged with MP3InfoExtension
      //FIXME v1.0 Added another if case when the header.size is bigger as
      //the whole id3header which is obviously wrong.
      if (head.getFrameID().equals("ZPAD") || head.getFrameID().equals("MP3e")
          || head.getSize() > this.header.getSize()) {
        DebugOut.println("Rest is padding");
        break;
      }
      //BUG v0.5 check for NegativeArraySizeException
      int newSize = head.getSize();
      //In case that The Size Field of the Tag is giving a number which is to
      // big
      if (newSize > tag.remaining() || newSize < 0) {
        newSize = tag.remaining();
      }

      frame = new byte[newSize];
      tag.get(frame);

      frames.putFrame(createFrame(ByteBuffer.wrap(frame), head));
    }
    return frames;

  }

}