/*
 * $Id: NoEmptyMetaData.java,v 1.2 2005/03/02 08:13:18 gruni Exp $ 
 * $Author: gruni $
 * $Revision: 1.2 $
 * $Date: 2005/03/02 08:13:18 $
 * $Log: NoEmptyMetaData.java,v $
 * Revision 1.2  2005/03/02 08:13:18  gruni
 * null pointer exception catch
 *
 * Revision 1.1  2005/02/04 08:32:02  gruni
 * new metadata class decorating metadata so that return values are never empty or null
 *
 */
package org.jd3lib;

/**
 * @author Andreas Grunewald
 * 
 * Class which changes empty or null valued strings to a return value "Unknown"
 * this helps when using Metadata to generate lists, avoiding nullpointer
 * exceptions or just empty fields. The Class also removes whitespace at the end
 * when return values
 */
public class NoEmptyMetaData implements MetaData {

  private MetaData dec;
  private static final String UNKNOWN = "(unknown)";

  public NoEmptyMetaData(MetaData input) {
    if (input != null){
      dec = input;
    } else throw new RuntimeException("You have to give MetaData otheriwse this doesn't work");
  }

  /**
   * @param title
   * @return
   */
  private static String unknow(String out) {
    if (out == null || out.equals("")) return UNKNOWN;
    return out.trim();
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.MetaData#getTitle()
   */
  public String getTitle() {
    return unknow(dec.getTitle());
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.MetaData#setTitle(java.lang.String)
   */
  public void setTitle(String newTitle) {
    dec.setTitle(newTitle);
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.MetaData#getArtist()
   */
  public String getArtist() {
    return unknow(dec.getArtist());
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.MetaData#setArtist(java.lang.String)
   */
  public void setArtist(String newArtist) {
    dec.setArtist(newArtist);
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.MetaData#getAlbum()
   */
  public String getAlbum() {
    return unknow(dec.getAlbum());
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.MetaData#setAlbum(java.lang.String)
   */
  public void setAlbum(String newAlbum) {
    dec.setAlbum(newAlbum);
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.MetaData#getYear()
   */
  public int getYear() {
    //TODO 0.9 check if really no NULL value possible
    return dec.getYear();
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.MetaData#setYear(int)
   */
  public void setYear(int newYear) {
    dec.setYear(newYear);
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.MetaData#getComment()
   */
  public String getComment() {
    //TODO 1.0 change behaviour to maybe return empty string instead of unknown
    return unknow(dec.getComment());
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.MetaData#setComment(java.lang.String)
   */
  public void setComment(String newComment) {
    dec.setComment(newComment);
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.MetaData#getTrack()
   */
  public int getTrack() {
    //TODO 0.9 check if really no NULL value possible
    return dec.getTrack();
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.MetaData#setTrack(int)
   */
  public void setTrack(int newTrack) {
    dec.setTrack(newTrack);
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.MetaData#getGenre()
   */
  public String getGenre() {
    return unknow(dec.getGenre());
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.MetaData#setGenre(java.lang.String)
   */
  public void setGenre(String newGenre) {
    dec.setGenre(newGenre);

  }

}