/*
 * $Id: Files.java,v 1.9 2005/04/27 08:13:20 gruni Exp $
 * $Author: gruni $ 
 * $Revision: 1.9 $
 * $Date: 2005/04/27 08:13:20 $
 * $Log: Files.java,v $
 * Revision 1.9  2005/04/27 08:13:20  gruni
 * -new logging framework
 * -wma support
 * -extended javadoc
 *
 */
package org.jd3lib.archoslib;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Iterator;
import java.util.Vector;

import org.jd3lib.compatible.Logger;

/**
 * <p>
 * 2 - Files with num_file items.
 * </p>
 * 
 * <p>
 * The file structure holds all information of one file in the media library and
 * points to an actual file on the hard drive:
 * </p>
 * 
 * <pre><code>
 *         struct file_str {
 *           
 *           path_t path; pointer to the file path
 *           
 *           string_t name; file name on hard drive
 *           
 *           string_t artist; artist name (for music)
 *           
 *           string_t album; album name (for music)
 *           
 *           string_t title; song title (for music)
 *           
 *           uint8_t flags; flags to be used by Device, must be set to 0!
 *           
 *           uint8_t track; track # (on CD, record, for music)
 *           
 *           uint8_t type; file type
 *           
 *           uint8_t genre; file genre
 *           
 *           uint16_t year; file year
 *           
 *           uint16_t reserved; must be set to 0! 
 *         }</code></pre>
 * 
 * <p>
 * If a file has no path (i.e. its in the root folder of the hard drive) the
 * path value is &quot;-1&quot;</br> If a text string for artist, album or
 * title is not set, the value of the pointer is &quot;-1&quot;
 * </p>
 * <p>
 * Flags are used by the Multimedia Device during operation and must be set to 0
 * when the library is created.
 * </p>
 * <p>
 * The track # can be used to sort the songs in a list not only alphabetically
 * but also according to the actual track sequence on the CD. The track # is
 * limited to a maximum 255!
 * </p>
 * <p>
 * Currently these values for type are defined:</br>
 * <pre>
 *          0 MP3 file &quot;.mp3&quot;
 *          1 MP2 file &quot;.mp2&quot;
 *          2 WAV-PCM file &quot;.wav&quot;
 *          3 WMA file &quot;.wma&quot;
 * </pre></p>
 * <p>
 * As the actual filename extensions are not stored in the library it is
 * imperative that each media file has the correct type set in order to be able
 * to find it on the hard drive.
 * </p>
 * <p>
 * <code>genre</code> specifies the music genre according to the extended
 * Winamp genre list as specified in appendix A.
 * </p>
 * <p>
 * <code>year</code> gives the recording/creation year.
 * </p>
 * <p>
 * Reserved is reserved for future use and should be set to 0!
 * </p>
 * 
 * @author Grunewald
 */
public class Files
    extends Section implements ArcFileProcessor {

  public static final byte MP3 = 0;// MP3 file ".mp3"

  public static final byte MP2 = 1;// MP2 file ".mp2"

  public static final byte WAV_PCM = 2;// WAV-PCM file ".wav"

  public static final byte WMA = 3;// WMA file ".wma"

  private Vector files = new Vector();

  private Strings strings;

  private Paths paths;

  public Files(Strings strs, Paths pts) {
    strings = strs;
    paths = pts;
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.archoslib.Section#toByteBuffer()
   */
  protected ByteBuffer toByteBuffer() {
    //TODO 2.0 externalize SingleFileSize of 28
    ByteBuffer rest = ByteBuffer.allocate(files.size() * 28);
    System.out.println("Number of Files:" + files.size());
    Iterator theFiles = files.iterator();
    while (theFiles.hasNext()) {
      Logger.global.finest("IN WHILE"); //TODO REMOVE LOG
      rest.put(this.getBufferForFile((ArcAudioFileDecorator) theFiles.next()));

    }
    Logger.global.finest("END FILE toByteBuffer"); //TODO REMOVE LOG
    rest.flip();

    return rest;
  }

  /**
   * @return
   */
  private ByteBuffer getBufferForFile(ArcAudioFileDecorator in) {
    int path = paths.get(in.getTheFile().getParentFile());//lib.paths.get();
    int name = strings.get(in.getFileName());
    int artist = strings.get(in.getMetaData().getArtist());
    int album = strings.get(in.getMetaData().getAlbum());
    int title = strings.get(in.getMetaData().getTitle());
    byte flags = 0;
    byte track = (byte) in.getMetaData().getTrack();
    //TODO refactor is bad workaround code
    String temp = in.getFileType().toLowerCase();
    byte type = MP3;
    if (temp.equals(".mp3"))
      type = MP3;
    else if (temp.equals(".wma")) type = WMA;
    byte genre = 0;
    short year = (short) in.getMetaData().getYear();
    short reserved = 0;
    ByteBuffer ret = ByteBuffer.allocate(28);
    ret.order(ByteOrder.LITTLE_ENDIAN);
    // System.out.println("ByteOrder="+ret.order());
    ret.putInt(path);//path_t path; pointer to the file path
    ret.putInt(name);//string_t name; file name on hard drive
    ret.putInt(artist);//string_t artist; artist name (for music)
    ret.putInt(album);//string_t album; album name (for music)
    ret.putInt(title);//string_t title; song title (for music)
    ret.put(flags);//uint8_t flags; flags to be used by Device, must be
    // set to 0!
    ret.put(track);//uint8_t track; track # (on CD, record, for music)
    ret.put(type);//uint8_t type; file type
    ret.put(genre);//uint8_t genre; file genre
    ret.putShort(year);//uint16_t year; file year
    ret.putShort(reserved);//uint16_t reserved; must be set to 0!*/
    ret.flip();
    return ret;
  }

  /**
   * @return
   */
  protected int size() {
    return files.size();
  }

  /**
   * Takes the File and adds it to the Vector of Files
   * 
   * @param data
   */
  public void take(ArcAudioFileDecorator data) {
    data.setNumber((short) files.size());
    files.add(data);

  }

}