/*
 * Created on 31.10.2004
 */
package org.jd3lib.archoslib;

import java.io.File;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

//TODO v2.0 Reimport Queue and LinkedList
//when switching to JDK 1.5 after release of JDK for MacOS
//import java.util.Queue;
//import java.util.LinkedList;
/**
 * The library consists of these sections: <table><tr><td>1. Header</td>
 * <td>struct head_str head</td></tr><tr><td>2. Files with num_files items
 * </td><td>struct files_str files[num_files]</td><tr><tr><td>3. Lists
 * with num_list items</td><td>struct list_str lists[num_lists]</td></tr>
 * <tr><td>4. List entries for all lists</td><td>uint16_t list_entries[ ... ]
 * </td></tr><tr><td>5. File path information for all files</td><td>
 * uint8_t paths[ total paths size ]</td></tr><tr><td>6. String data for
 * files, lists, paths</td><td>uint8_t strings[ total string size ]</td>
 * </tr><tr><td>7. Private data  if present</td><td>arbitrary structure
 * and size</td></tr> </table> <p>In order to keep the size of the library
 * file small so that it can be held completely in RAM of the players, the
 * number of files and lists (not list entries) together is limited to 65536 (16
 * bit).</p><p>The maximum size of the library file is 1MB for Gmini220 and
 * 2MB for Gmini120 products.</p><p>"Files" and "Lists" share the same
 * numbering, "lists" are numbered from 0 to (num_list  1) and "lists" are
 * numbered from num_list to (num_list + num_file  1)</p><p>For the
 * following structures these data types are used: <dl><dt>string_t:</dt>
 * <dd>uint32_t, byte offset into strings[]</dd><dt>path_t:</dt><dd>
 * uint32_t, byte offset into paths[]</dd><dt>file_list_t:</dt><dd>
 * uint16_t, number of file or list, used as index into lists or files array.
 * </dd></dl></p><p>All sections must be aligned to the next 512 byte (Hard
 * drive sector size) boundary.</p>
 * 
 * <p>All structures must be packed, 8bit and 16bit size values must be
 * packed into 32bit words.</p>
 * 
 * <p>All data is stored in little endian mode</p>
 * 
 * @author Grunewald
 * @version 0.3
 */
public class LibraryGUI implements ChangeListener {

  private LibraryWriter theLib;

  private DirectoryTraverser theTrav;

  /*
   * TODO v2.0 Exchange the ProgBar with a general percentage so that the GUI
   * can be externalised.
   */
  JProgressBar progBar;

  public LibraryGUI(String workDirectory) {
    theLib = new LibraryWriter(new File(workDirectory));
    createAndShowGUI();
    theTrav = new DirectoryTraverser();
    theTrav.addChangeListener(this);
    theLib.addChangeListener(this);

  }

  /**
   * Iterates over the Found MP3s and inserts the MetaData into the Strings and
   *  
   */

  /**
   * Create the GUI and show it. For thread safety, this method should be
   * invoked from the event-dispatching thread.
   */
  private void createAndShowGUI() {
    //Make sure we have nice window decorations.
    JFrame.setDefaultLookAndFeelDecorated(true);

    //Create and set up the window.
    JFrame frame = new JFrame("ARClib Progress");

    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    //Create and set up the content pane.
    JComponent newContentPane = new JPanel();
    JProgressBar pBar = new JProgressBar(0, 100);
    pBar.setStringPainted(true);
    progBar = pBar;
    //Construct the tree.
    //JTree tree = new GenealogyTree(GenealogyExample.getGenealogyGraph());
    //JScrollPane scrollPane = new JScrollPane(tree);
    //scrollPane.setPreferredSize(new Dimension(200, 200));

    newContentPane.add(new JLabel("Files processed:"));
    newContentPane.add(pBar);
    //newContentPane.add(scrollPane);
    newContentPane.setOpaque(true); //content panes must be opaque

    frame.setContentPane(newContentPane);

    //Display the window.
    frame.pack();
    frame.setVisible(true);
  }

  /* (non-Javadoc)
   * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
   */
  public void stateChanged(ChangeEvent e) {
    //
    if (e.getSource() == theLib) {
      this.progBar.setValue(theLib.getPercentage());
      this.progBar.setString(theLib.getProcessed() + "/"
          + theLib.getFileCount() + " [" + theLib.getPercentage() + "%]");
    } else if (e.getSource() == theTrav)
      this.progBar.setString(theTrav.getFoundFiles() + " files found");
    else
      this.progBar.setString(e.getSource().toString());

  }

}