/*
 * $Id: ListEntries.java,v 1.8 2005/04/27 08:19:12 gruni Exp $
 * $Author: gruni $ 
 *
 * $Log: ListEntries.java,v $
 * Revision 1.8  2005/04/27 08:19:12  gruni
 * minor changes in documentation
 *
 */
package org.jd3lib.archoslib;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Iterator;
import java.util.Vector;

import org.jd3lib.archoslib.lists.ArcEntry;
import org.jd3lib.archoslib.lists.List;
import org.jd3lib.archoslib.lists.ListAsRoot;
import org.jd3lib.util.Coder;

/**
 * <p>4 - List entries for all lists.</p>
 * 
 * <p>As the number of files and lists is limited to 16bit, the actual list
 * entries are just an array of unit16_t. The <i>offset </i> pointer of struct
 * <i>list_str </i> gives the position of the first list entry in this array.
 * </p>
 * 
 * <p>List entries are either references to further lists or media files. In
 * case of lists, the list entries give the number of the list ( <i>num_list
 * </i> to ( <i>num_list + num_file - 1 </i>) ), in case of files, the list
 * entries give the index number ( <i>0 </i> to ( <i>num_list - 1 </i>) ) of the
 * files defined in <i>struct files_str files[] </i>.</p>
 * 
 * @author Grunewald
 */
public class ListEntries
    extends Section {

  /**
   * Vector&lt;ListEntryT&gt; containing the Lists with their entries as arrays
   * of shorts.
   */
  private Vector EntryList;

  /**
   * @author Grunewald
   */
  private class ListEntryT {
    private short[] listPointers;

    private int offset;

    public ListEntryT(short[] listEntryPointers, int offset_in) {
      listPointers = listEntryPointers;
      offset = offset_in;
    }

    public String toString() {
      String out = "Entry[Pos=" + offset + " ";
      for (int i = 0; i < listPointers.length; i++) {
        out += "(" + listPointers[i] + ")";
      }
      out += "]";
      return out;

    }

    public int getOffset() {
      return offset;
    }

    public ByteBuffer toByteBuffer() {
      ByteBuffer out = ByteBuffer.allocate(getSize() * 2);
      out.order(ByteOrder.LITTLE_ENDIAN);
      for (int i = 0; i < listPointers.length; i++) {
        out.putShort(listPointers[i]);
      }
      out.flip();
      return out;
    }

    /**
     * @return
     */
    public short getSize() {
      return (short) ((listPointers.length));
    }

  }

  //ByteBuffer lists;

  public ListEntries() {
    EntryList = new Vector(5, 5);
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.archoslib.Section#toByteBuffer()
   */
  protected ByteBuffer toByteBuffer() {
    //Calculation for Buffer allocation of next block by using the size
    //of the offset of the last Element and multiplying it with two because
    //of short with 2bytes
    //TODO v2.0 get next blockalign is not really neccecary check if it can be
    // deleted
    ByteBuffer rest = ByteBuffer.allocate(Coder
        .getNextBlockAlign(getOffset(EntryList.size()) * 2));
    rest.order(ByteOrder.LITTLE_ENDIAN);
    for (int i = 0; i < EntryList.size(); i++) {
      rest.put(((ListEntryT) EntryList.get(i)).toByteBuffer());

    }
    rest.flip();
    return rest;
  }

  /**
   * Calculates Offset incremental by obtaining the offset and size of the
   * previous element.
   * 
   * @param count2
   * @return
   */
  private int getOffset(int thePos) {
    int out = 0;
    if (thePos > 0) {
      thePos = thePos - 1; //didn't want to make thePos--
      ListEntryT temp = (ListEntryT) EntryList.get(thePos);
      out = temp.getOffset();
      out += temp.getSize();
    }
    return out;
  }

  /**
   * Puts this array of shorts into the Entrylist
   * 
   * @return the Offset to this list of Entries;
   */
  public int put(short[] inputList) {
    ListEntryT EList = new ListEntryT(inputList, getOffset(EntryList.size()));
    // System.out.println(EList);
    EntryList.add(EList);
    return EList.getOffset();
  }

  /**
   * Takes the Lists and Files and creates the ListEntries from it. Should only
   * be called after Lists and Files are completely read.
   * 
   * @param lists
   */
  public void take(Lists lists, Files files) {
    int num_files = files.size();
    ListAsRoot root = lists.getRootList();
    Iterator ote = root.getVectorOfAllLists().iterator();
    Vector currentEntries;
    short[] currentListOfEntries;
    List current;
    ArcEntry innerEnt;
    while (ote.hasNext()) {
      current = (List) (ote.next());
      currentEntries = current.getEntries();
      currentListOfEntries = new short[currentEntries.size()];
      for (int i = 0; i < currentListOfEntries.length; i++) {
        innerEnt = ((ArcEntry) currentEntries.get(i));
        if (innerEnt instanceof List)
          currentListOfEntries[i] = (short) (innerEnt.getNumber() + num_files);
        else
          currentListOfEntries[i] = innerEnt.getNumber();

      }
      current.setOffset(put(currentListOfEntries));
    }
    System.out.println("done taking lists");
  }

}