/*
 * $Id: WMAFile.java,v 1.2 2005/04/27 08:26:09 gruni Exp $
 * $Author: gruni $ 
 * 
 * $Log: WMAFile.java,v $
 * Revision 1.2  2005/04/27 08:26:09  gruni
 * -corrected Number format exception bug
 * -correct getFileName it returns not extension now.
 *
 * Revision 1.1  2005/04/04 15:19:38  gruni
 * WMA file support uses entagged
 *
 */
package org.jd3lib.archoslib;

import java.io.File;

import org.jd3lib.AudioFile;
import org.jd3lib.MetaData;
import org.jd3lib.compatible.Logger;

import entagged.audioformats.AudioFileIO;
import entagged.audioformats.Tag;
import entagged.audioformats.exceptions.CannotReadException;

/**
 * 
 * @version 1.5 MetaData parsing of tracknumber won't throw a number format
 *          exception and cause the programm to crash
 * @author Grunewald
 */
public class WMAFile implements AudioFile {
  
  private class WMAData implements MetaData {
    
    Tag data;
    
    /**
     * @param tag
     */
    public WMAData(Tag tag) {
      data = tag;
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.jd3lib.MetaData#getTitle()
     */
    public String getTitle() {
      return data.getFirstTitle();
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.jd3lib.MetaData#setTitle(java.lang.String)
     */
    public void setTitle(String newTitle) {
      data.setTitle(newTitle);
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.jd3lib.MetaData#getArtist()
     */
    public String getArtist() {
      return data.getFirstArtist();
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.jd3lib.MetaData#setArtist(java.lang.String)
     */
    public void setArtist(String newArtist) {
      data.setArtist(newArtist);
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.jd3lib.MetaData#getAlbum()
     */
    public String getAlbum() {
      return data.getFirstAlbum();
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.jd3lib.MetaData#setAlbum(java.lang.String)
     */
    public void setAlbum(String newAlbum) {
      data.setAlbum(newAlbum);
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.jd3lib.MetaData#getYear()
     */
    public int getYear() {
      String work = data.getFirstTrack();
      int out = -1;
      try {
        out = Integer.parseInt(work);
      } catch (NumberFormatException e) {
        e.printStackTrace();
        Logger.global.info("Couldn't parse the Year it was [" + work + "]");
        out = 0;
      }
      return out;
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.jd3lib.MetaData#setYear(int)
     */
    public void setYear(int newYear) {
      data.setYear(String.valueOf(newYear));
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.jd3lib.MetaData#getComment()
     */
    public String getComment() {
      return data.getFirstComment();
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.jd3lib.MetaData#setComment(java.lang.String)
     */
    public void setComment(String newComment) {
      data.setComment(newComment);
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.jd3lib.MetaData#getTrack()
     */
    public int getTrack() {
      String work = data.getFirstTrack();
      int out = -1;
      try {
        out = Integer.parseInt(work);
      } catch (NumberFormatException e) {
        //TODO Make trace available somehow e.printStackTrace();
        Logger.global.info("Couldn't parse the Tracknumber it was [" + work
            + "]");
        if (work.length() > 0 && work.charAt(0) == '#') {
          try {
            out = Integer.parseInt(work.substring(1));
          } catch (NumberFormatException e2) {
            Logger.global.info("Ok I tried parsing [" + work.substring(1)
                + "] still\n" + "doesn't seem like a number");
          }
        }
      }
      return out;
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.jd3lib.MetaData#setTrack(int)
     */
    public void setTrack(int newTrack) {
      data.setTrack(String.valueOf(newTrack));
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.jd3lib.MetaData#getGenre()
     */
    public String getGenre() {
      return data.getFirstGenre();
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.jd3lib.MetaData#setGenre(java.lang.String)
     */
    public void setGenre(String newGenre) {
      data.setGenre(newGenre);
    }
    
  }
  
  private File theFile;
  
  private WMAData theMeta;
  
  /**
   * @param input
   */
  public WMAFile(File input) {
    theFile = input;
    
    try {
      entagged.audioformats.AudioFile myFile = AudioFileIO.read(input);
      theMeta = new WMAData(myFile.getTag());
    } catch (CannotReadException e) {
      e.printStackTrace();
    }
  }
  
  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.AudioFile#getMetaData()
   */
  public MetaData getMetaData() {
    return theMeta;
  }
  
  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.AudioFile#getTheFile()
   */
  public File getTheFile() {
    return theFile;
  }
  
  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.AudioFile#getFileName()
   */
  public String getFileName() {
    return theFile.getName().substring(0, theFile.getName().lastIndexOf('.'));
  }
  
  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.AudioFile#getFileType()
   */
  public String getFileType() {
    //TODO v1.1 selber Code wie MP3File code duplication is bad
    return theFile.getName().substring(theFile.getName().lastIndexOf('.'),
        theFile.getName().length());
  }
  
}