/*
 * 22.08.2004 by Andreas Grunewald
 */
package org.jd3lib.util;

import java.nio.ByteBuffer;

/**
 * @author Andreas Grunewald
 * 
 * LATER 1.0 Write Documentation
 */
public final class Coder {



  /**
   * This function returns the ID3Header Size given the 4 bytes containing the
   * size in bytes
   * 
   * @param sizeArray
   *          A byte array of size 4 containing the byte 6,7,8,9 (Headerbytes
   *          are 0 to 9) of the ID3v2 Header
   * @return The full ID3v2 Tagsize including the 10 bytes for header. The
   *         returned number should be the first byte of the MP3 File.
   * @deprecated First Algorith compareably slow due to loop.
   */
  public static int getSyncsafeIntegerValueOld(byte sizeArray[]) {
    int out = 0;
    /*
     * Calculates the header size by multiplying each byte with 2^7*bytenumber
     * because only the first 7 bit of the byte are used to have a syncsafe
     * integer.
     */
    for (int i = 0; i < sizeArray.length; i++) {
      if (sizeArray[i] > 0) out += (int) (sizeArray[i] * Math.pow(2,
          7 * ((sizeArray.length - 1) - i)));
    }
    return out;
  }

  /**
   * This function returns the ID3Header Size given the 4 bytes containing the
   * size in bytes
   * 
   * @param sizeArray
   *          A byte array of size 4 containing the byte 6,7,8,9 (Headerbytes
   *          are 0 to 9) of the ID3v2 Header
   * @return The full ID3v2 Tagsize including the 10 bytes for header. The
   *         returned number should be the first byte of the MP3 File.
   */
  public static int getSyncsafeIntegerValue(byte sizeArray[]) {
    return (sizeArray[0] << 21) + (sizeArray[1] << 14) + (sizeArray[2] << 7)
        + sizeArray[3];
  }

  /**
   * This function returns the ID3v2.3 Frame Size by claculation a byte[4] into
   * an int.
   * 
   * @param temp
   *          32bit Integervalue put into a 4 byte array
   * @return the integer representation.
   */
  public static int getIntegerValue(byte[] temp) {
    // As a matter of fact if you use a full32bit size field as it is in the
    // frames of ID3v2.3 then you can be sure because of the limitation of the
    // ID3 Header itself and it's syncsafe integers that you never get into
    // negative values on the byte[0]
    return (temp[0] << 24) + ((temp[1] << 24) >>> 8) + ((temp[2] << 24) >>> 16)
        + ((temp[3] << 24) >>> 24);
    /*return (temp[0] << 24) + (temp[1] << 16) + (temp[2] << 8)
    + temp[3];*/

  }
  
  
  /**
   * Converts int to an array of 4 bytes each with only the first 7 bits filled.
   * 
   * @param size
   *          the int Value max 28bit to be converted
   * @return
   */
  public static byte[] getSyncsafeBytes(int size) {
    byte[] out = { (byte) ((size << 4) >>> 25), (byte) ((size << 11) >>> 25),
        (byte) ((size << 18) >>> 25), (byte) ((size << 25) >>> 25) };
    return out;
  }

  /**
   * @param value
   * @return
   */
  public static byte[] getBytes(int value) {
    byte[] out = { (byte) (value >>> 24), (byte) ((value << 8) >>> 24),
        (byte) ((value << 16) >>> 24), (byte) ((value << 24) >>> 24) };
    return out;
  }
  
  public static ByteBuffer getByteBuffer(int value){
      ByteBuffer out = ByteBuffer.wrap(getBytes(value));
      return out;
  }
  
  /**
   * Computes the next 512*n which is bigger than value
   * */
  public static int getNextBlockAlign(int value){
      
    return 512+((value-1)/512)*512;
  }
}

